package org.jeecg.modules.system.app.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.subject.Subject;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.system.app.dto.FindPageRequestDTO;
import org.jeecg.modules.system.app.dto.FindResponseDTO;
import org.jeecg.modules.system.app.entity.AppFeedback;
import org.jeecg.modules.system.app.enums.ReplyStatusEnum;
import org.jeecg.modules.system.app.service.IAppFeedbackService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Date;

/**
 * @Description: 意见反馈
 * @Author: zx
 * @Date:   2025-07-07
 * @Version: V1.0
 */
@Tag(name="意见反馈")
@RestController
@RequestMapping("/appFeedback")
@Slf4j
public class FeedbackController extends JeecgController<AppFeedback, IAppFeedbackService> {
	@Autowired
	private IAppFeedbackService appFeedbackService;
	
	/**
	 * 分页列表查询
	 *
	 * @param findPageRequestDTO
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@Operation(summary="意见反馈-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<FindResponseDTO>> queryPageList(FindPageRequestDTO findPageRequestDTO,
														@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														HttpServletRequest req) {


		IPage<FindResponseDTO> pageList = appFeedbackService.findPage(pageNo,pageSize, findPageRequestDTO);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param appFeedback
	 * @return
	 */
	@AutoLog(value = "意见反馈-添加")
	@Operation(summary="意见反馈-添加")
	@RequiresPermissions("feedback:nm_feedback:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody AppFeedback appFeedback) {
		appFeedbackService.save(appFeedback);

		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param appFeedback
	 * @return
	 */
	@AutoLog(value = "意见反馈-编辑")
	@Operation(summary="意见反馈-编辑")
	@RequiresPermissions("feedback:nm_feedback:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody AppFeedback appFeedback) {
		Subject subject = SecurityUtils.getSubject();
		LoginUser loginUser =null;
		if (subject.isAuthenticated()) {
			// 获取当前登录的用户信息
			loginUser= (LoginUser) subject.getPrincipal();
		}
		if (loginUser==null){
			return Result.error("当前未登录,请先登录;或者未获取到用户信息，请联系管理员");
		}
		appFeedback.setReplyStatus(ReplyStatusEnum.REPLIED.getCode());
		appFeedback.setReplyUserId(loginUser.getId());
		appFeedback.setUpdateTime(new Date());
		appFeedback.setUpdateBy(loginUser.getId());
		appFeedbackService.updateById(appFeedback);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "意见反馈-通过id删除")
	@Operation(summary="意见反馈-通过id删除")
	@RequiresPermissions("feedback:nm_feedback:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		appFeedbackService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "意见反馈-批量删除")
	@Operation(summary="意见反馈-批量删除")
	@RequiresPermissions("feedback:nm_feedback:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.appFeedbackService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "意见反馈-通过id查询")
	@Operation(summary="意见反馈-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<FindResponseDTO> queryById(@RequestParam(name="id",required=true) String id) {
		FindResponseDTO findResponseDTO = appFeedbackService.FindById(id);
		if(findResponseDTO==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(findResponseDTO);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param appFeedback
    */
    @RequiresPermissions("feedback:nm_feedback:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, AppFeedback appFeedback) {
        return super.exportXls(request, appFeedback, AppFeedback.class, "意见反馈");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("feedback:nm_feedback:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, AppFeedback.class);
    }

}
